# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.steady_wake_deficit import SteadyWakeDeficit

from .schema_helper import SchemaHelper
from .models_impl import *


class Gaussian(SteadyWakeDeficit):
    r"""
    A simple model for the reduced velocity of the wind behind another turbine.  This deficit will be applied across a certain region, and this region will not move around during the simulation.  The velocity deficit profile will be based on a Gaussian distribution, scaled by a maximum deficit and the radius of the region affected.
    
    Attributes
    ----------
    SteadyWakeDeficitType : Literal['Gaussian'], default='Gaussian'
        Defines the specific type of SteadyWakeDeficit model in use.  For a `Gaussian` object, this must always be set to a value of `Gaussian`.
    
    CentrelineVelocityDeficit : float
        The maximum deficit within the affected region, occurring in the centre.  1.0 or 100% would be a reduction of the wind to zero, whereas 0.0 or 0% would represent no change in the free-field wind speed.
    
    WidthOfDeficit : float
        The width of the region within which there is a velocity deficit.
    
    Notes
    -----
    
    """
    SteadyWakeDeficitType: Literal['Gaussian'] = Field(alias="SteadyWakeDeficitType", default='Gaussian', allow_mutation=False, const=True) # type: ignore
    CentrelineVelocityDeficit: float = Field(alias="CentrelineVelocityDeficit", default=None)
    WidthOfDeficit: float = Field(alias="WidthOfDeficit", default=None)

    _relative_schema_path = 'TimeDomainSimulation/Environment/Wind/SteadyWakeDeficit/Gaussian.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'SteadyWakeDeficitType').merge(SteadyWakeDeficit._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


Gaussian.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.aerodynamic_information_calculation import AerodynamicInformationCalculation
from dnv_bladed_models.blade_stability_analysis import BladeStabilityAnalysis
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.campbell_diagram import CampbellDiagram
from dnv_bladed_models.constants import Constants
from dnv_bladed_models.modal_analysis_calculation import ModalAnalysisCalculation
from dnv_bladed_models.model_linearisation import ModelLinearisation
from dnv_bladed_models.performance_coefficients_calculation import PerformanceCoefficientsCalculation
from dnv_bladed_models.settings import Settings
from dnv_bladed_models.steady_calculation import SteadyCalculation
from dnv_bladed_models.steady_calculation_insert import SteadyCalculationInsert
from dnv_bladed_models.steady_operational_loads_calculation import SteadyOperationalLoadsCalculation
from dnv_bladed_models.steady_parked_loads_calculation import SteadyParkedLoadsCalculation
from dnv_bladed_models.steady_power_curve_calculation import SteadyPowerCurveCalculation
from dnv_bladed_models.time_domain_simulation import TimeDomainSimulation
from dnv_bladed_models.turbine import Turbine

from .schema_helper import SchemaHelper
from .models_impl import *

TSteadyCalculationOptions = TypeVar('TSteadyCalculationOptions', AerodynamicInformationCalculation, BladeStabilityAnalysis, CampbellDiagram, SteadyCalculationInsert, ModalAnalysisCalculation, ModelLinearisation, PerformanceCoefficientsCalculation, SteadyOperationalLoadsCalculation, SteadyParkedLoadsCalculation, SteadyPowerCurveCalculation, SteadyCalculation, )

class BladedAnalysis(BladedModel):
    r"""
    The definition of a single Bladed analysis.
    
    Attributes
    ----------
    TimeDomainSimulation : TimeDomainSimulation
    
    SteadyCalculation : Union[AerodynamicInformationCalculation, BladeStabilityAnalysis, CampbellDiagram, SteadyCalculationInsert, ModalAnalysisCalculation, ModelLinearisation, PerformanceCoefficientsCalculation, SteadyOperationalLoadsCalculation, SteadyParkedLoadsCalculation, SteadyPowerCurveCalculation]
    
    Settings : Settings
    
    Constants : Constants
    
    Turbine : Turbine
    
    Notes
    -----
    
    """
    TimeDomainSimulation: TimeDomainSimulation = Field(alias="TimeDomainSimulation", default=None)
    SteadyCalculation: Union[AerodynamicInformationCalculation, BladeStabilityAnalysis, CampbellDiagram, SteadyCalculationInsert, ModalAnalysisCalculation, ModelLinearisation, PerformanceCoefficientsCalculation, SteadyOperationalLoadsCalculation, SteadyParkedLoadsCalculation, SteadyPowerCurveCalculation] = Field(alias="SteadyCalculation", default=None, discriminator='SteadyCalculationType')
    Settings: Settings = Field(alias="Settings", default=None)
    Constants: Constants = Field(alias="Constants", default=None)
    Turbine: Turbine = Field(alias="Turbine", default=None)

    _relative_schema_path = 'BladedAnalysis.json'
    _type_info = TypeInfo(
        set([('SteadyCalculation', 'SteadyCalculationType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def SteadyCalculation_as_AerodynamicInformationCalculation(self) -> AerodynamicInformationCalculation:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a AerodynamicInformationCalculation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        AerodynamicInformationCalculation
            A model object, guaranteed to be a AerodynamicInformationCalculation.

        Raises
        ------
        TypeError
            If the value is not a AerodynamicInformationCalculation.
        """
        return self.SteadyCalculation_as(AerodynamicInformationCalculation)


    @property
    def SteadyCalculation_as_BladeStabilityAnalysis(self) -> BladeStabilityAnalysis:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a BladeStabilityAnalysis; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        BladeStabilityAnalysis
            A model object, guaranteed to be a BladeStabilityAnalysis.

        Raises
        ------
        TypeError
            If the value is not a BladeStabilityAnalysis.
        """
        return self.SteadyCalculation_as(BladeStabilityAnalysis)


    @property
    def SteadyCalculation_as_CampbellDiagram(self) -> CampbellDiagram:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a CampbellDiagram; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        CampbellDiagram
            A model object, guaranteed to be a CampbellDiagram.

        Raises
        ------
        TypeError
            If the value is not a CampbellDiagram.
        """
        return self.SteadyCalculation_as(CampbellDiagram)


    @property
    def SteadyCalculation_as_ModalAnalysisCalculation(self) -> ModalAnalysisCalculation:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a ModalAnalysisCalculation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ModalAnalysisCalculation
            A model object, guaranteed to be a ModalAnalysisCalculation.

        Raises
        ------
        TypeError
            If the value is not a ModalAnalysisCalculation.
        """
        return self.SteadyCalculation_as(ModalAnalysisCalculation)


    @property
    def SteadyCalculation_as_ModelLinearisation(self) -> ModelLinearisation:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a ModelLinearisation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ModelLinearisation
            A model object, guaranteed to be a ModelLinearisation.

        Raises
        ------
        TypeError
            If the value is not a ModelLinearisation.
        """
        return self.SteadyCalculation_as(ModelLinearisation)


    @property
    def SteadyCalculation_as_PerformanceCoefficientsCalculation(self) -> PerformanceCoefficientsCalculation:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a PerformanceCoefficientsCalculation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PerformanceCoefficientsCalculation
            A model object, guaranteed to be a PerformanceCoefficientsCalculation.

        Raises
        ------
        TypeError
            If the value is not a PerformanceCoefficientsCalculation.
        """
        return self.SteadyCalculation_as(PerformanceCoefficientsCalculation)


    @property
    def SteadyCalculation_as_SteadyOperationalLoadsCalculation(self) -> SteadyOperationalLoadsCalculation:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a SteadyOperationalLoadsCalculation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        SteadyOperationalLoadsCalculation
            A model object, guaranteed to be a SteadyOperationalLoadsCalculation.

        Raises
        ------
        TypeError
            If the value is not a SteadyOperationalLoadsCalculation.
        """
        return self.SteadyCalculation_as(SteadyOperationalLoadsCalculation)


    @property
    def SteadyCalculation_as_SteadyParkedLoadsCalculation(self) -> SteadyParkedLoadsCalculation:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a SteadyParkedLoadsCalculation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        SteadyParkedLoadsCalculation
            A model object, guaranteed to be a SteadyParkedLoadsCalculation.

        Raises
        ------
        TypeError
            If the value is not a SteadyParkedLoadsCalculation.
        """
        return self.SteadyCalculation_as(SteadyParkedLoadsCalculation)


    @property
    def SteadyCalculation_as_SteadyPowerCurveCalculation(self) -> SteadyPowerCurveCalculation:
        """
        Retrieves the value of SteadyCalculation guaranteeing it is a SteadyPowerCurveCalculation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        SteadyPowerCurveCalculation
            A model object, guaranteed to be a SteadyPowerCurveCalculation.

        Raises
        ------
        TypeError
            If the value is not a SteadyPowerCurveCalculation.
        """
        return self.SteadyCalculation_as(SteadyPowerCurveCalculation)


    @property
    def SteadyCalculation_as_inline(self) -> Union[AerodynamicInformationCalculation, BladeStabilityAnalysis, CampbellDiagram, ModalAnalysisCalculation, ModelLinearisation, PerformanceCoefficientsCalculation, SteadyOperationalLoadsCalculation, SteadyParkedLoadsCalculation, SteadyPowerCurveCalculation]:
        """
        Retrieves the value of SteadyCalculation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[AerodynamicInformationCalculation, BladeStabilityAnalysis, CampbellDiagram, ModalAnalysisCalculation, ModelLinearisation, PerformanceCoefficientsCalculation, SteadyOperationalLoadsCalculation, SteadyParkedLoadsCalculation, SteadyPowerCurveCalculation]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of SteadyCalculation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.SteadyCalculation, SteadyCalculationInsert) or self.SteadyCalculation.is_insert:
            raise TypeError(f"Expected SteadyCalculation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.SteadyCalculation


    def SteadyCalculation_as(self, cls: Type[TSteadyCalculationOptions])-> TSteadyCalculationOptions:
        """
        Retrieves the value of SteadyCalculation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of SteadyCalculation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[AerodynamicInformationCalculation, BladeStabilityAnalysis, CampbellDiagram, SteadyCalculationInsert, ModalAnalysisCalculation, ModelLinearisation, PerformanceCoefficientsCalculation, SteadyOperationalLoadsCalculation, SteadyParkedLoadsCalculation, SteadyPowerCurveCalculation]]
            One of the valid concrete types of SteadyCalculation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TSteadyCalculationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of SteadyCalculation:
        >>> val_obj = model_obj.SteadyCalculation_as(models.AerodynamicInformationCalculation)
        >>> val_obj = model_obj.SteadyCalculation_as(models.BladeStabilityAnalysis)
        >>> val_obj = model_obj.SteadyCalculation_as(models.CampbellDiagram)
        >>> val_obj = model_obj.SteadyCalculation_as(models.ModalAnalysisCalculation)
        >>> val_obj = model_obj.SteadyCalculation_as(models.ModelLinearisation)
        >>> val_obj = model_obj.SteadyCalculation_as(models.PerformanceCoefficientsCalculation)
        >>> val_obj = model_obj.SteadyCalculation_as(models.SteadyOperationalLoadsCalculation)
        >>> val_obj = model_obj.SteadyCalculation_as(models.SteadyParkedLoadsCalculation)
        >>> val_obj = model_obj.SteadyCalculation_as(models.SteadyPowerCurveCalculation)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.SteadyCalculation_as(models.SteadyCalculationInsert)
        """
        if not isinstance(self.SteadyCalculation, cls):
            raise TypeError(f"Expected SteadyCalculation of type '{cls.__name__}' but was type '{type(self.SteadyCalculation).__name__}'")
        return self.SteadyCalculation


    def _entity(self) -> bool:
        return True


BladedAnalysis.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.integrator import Integrator

from .schema_helper import SchemaHelper
from .models_impl import *


class RungeKuttaVariableStep(Integrator):
    r"""
    Settings for the Runge Kutta Variable Step integrator.
    
    Attributes
    ----------
    IntegratorType : Literal['RungeKuttaVariableStep'], default='RungeKuttaVariableStep'
        Defines the specific type of Integrator model in use.  For a `RungeKuttaVariableStep` object, this must always be set to a value of `RungeKuttaVariableStep`.
    
    InitialStep : float, default=0
        The recommended value is zero: the minimum time step will in fact be used.  A value larger than the minimum time step will speed up the initialisation of the simulation, but there is a risk of numerical problems if too large a value is used.
    
    Tolerance : float, default=0.005
        The tolerance for the variable step integrator: This parameter defines the precision of the simulation. All states are integrated to an error within the integration tolerance multiplied by the state magnitude at that step. A higher value can increase simulation speed but lower precision.  Fixed step integrators: When the \"Maximum number of iterations\" > 1, the integrator relative tolerance is used to control how many iterations are carried out when integrating the first order and prescribed second order states. Iterations are carried out until the maximum number of iterations is reached, or until the change in all first order and prescribed state derivatives between successive iterations is less than the relative tolerance multiplied by the state derivative absolute value.
    
    MinimumTimeStep : float, default=1.0E-7
        The minimum time step.  The simulation uses a 4/5th order Runge-Kutta variable time step method.  The time step will be reduced automatically if the specified tolerance is exceeded, until this minimum value is reached.
    
    MaximumTimeStep : float, default=1
        The maximum time step.  This should normally be the same as the output time step, although a smaller value might be useful in some cases if the output time step is particularly long.  A very small value for the minimum time step is recommended, such as 10^-8 s, to ensure that the accuracy of simulation is not constrained by this.  In special cases, increasing the minimum time step may speed up the simulation with little loss of accuracy, but it is advisable to check that the results are not significantly altered by doing this.  This situation may arise for example with a dynamic mode which is inactive because it is heavily damped.  It may be better to remove the mode completely.
    
    Notes
    -----
    
    """
    IntegratorType: Literal['RungeKuttaVariableStep'] = Field(alias="IntegratorType", default='RungeKuttaVariableStep', allow_mutation=False, const=True) # type: ignore
    InitialStep: float = Field(alias="InitialStep", default=None)
    Tolerance: float = Field(alias="Tolerance", default=None)
    MinimumTimeStep: float = Field(alias="MinimumTimeStep", default=None)
    MaximumTimeStep: float = Field(alias="MaximumTimeStep", default=None)

    _relative_schema_path = 'Settings/SolverSettings/Integrator/RungeKuttaVariableStep.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'IntegratorType').merge(Integrator._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


RungeKuttaVariableStep.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.time_domain_current import TimeDomainCurrent
class TurbulentCurrent_CentreTurbulenceFileOnEnum(str, Enum):
    CENTRED_ON_HUB = "CENTRED_ON_HUB"
    BEST_FIT = "BEST_FIT"

from .schema_helper import SchemaHelper
from .models_impl import *


class TurbulentCurrent(TimeDomainCurrent):
    r"""
    The definition of a turbulent flow field, with the values for the turbulence defined in an external file.
    
    Not supported yet.
    
    Attributes
    ----------
    CurrentType : Literal['TurbulentCurrent'], default='TurbulentCurrent', Not supported yet
        Defines the specific type of Current model in use.  For a `TurbulentCurrent` object, this must always be set to a value of `TurbulentCurrent`.
    
    MeanSpeed : float, Not supported yet
        The mean current speed upon which the turbulence will be added.  This must correspond with the mean current speed used to create the turbulence file.
    
    TurbulenceFilepath : str, Not supported yet
        The filepath or URI of the turbulence file.
    
    TurbulenceIntensity : float, Not supported yet
        The turbulence intensity in the longitudinal (global X) direction.  This is used to scale the turbulence provided in the file.
    
    TurbulenceIntensityLateral : float, Not supported yet
        The turbulence intensity in the lateral (global Y) direction.  This is typically in the order of 80% of the longitudinal turbulence intensity.
    
    TurbulenceIntensityVertical : float, Not supported yet
        The turbulence intensity in the vertical (global Z) direction.  This is typically in the order of 50% of the longitudinal turbulence intensity.
    
    CentreTurbulenceFileOn : TurbulentCurrent_CentreTurbulenceFileOnEnum, Not supported yet
        The method used to position the data in the turbulence file relative to the turbine.  If any part of the rotor exceeds this box, the simulation will terminate with an exception.
    
    CentreTurbulenceFileAtHeight : float, Not supported yet
        The height at which to centre the data in the turbulence file.  If any part of the rotor exceeds this box, the simulation will terminate with an exception.
    
    RepeatTurbulenceFile : bool, Not supported yet
        If true, the turbulence file will be \"looped\".  If false, the turbulence will be 0 in all three components once the end of the file has been reached.  Using part of a turbulence file may invalidate its turbulence statistics, and no effort is made by Bladed to ensure coherence at the point when it transitions from the end of the wind file back to the beginning.
    
    TurbulenceFileStartTime : float, default=0, Not supported yet
        The time into turbulent wind file at start of simulation.  This can be used to synchronise the wind file with simulation.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    CurrentType: Literal['TurbulentCurrent'] = Field(alias="CurrentType", default='TurbulentCurrent', allow_mutation=False, const=True) # Not supported yet # type: ignore
    MeanSpeed: float = Field(alias="MeanSpeed", default=None) # Not supported yet
    TurbulenceFilepath: str = Field(alias="TurbulenceFilepath", default=None) # Not supported yet
    TurbulenceIntensity: float = Field(alias="TurbulenceIntensity", default=None) # Not supported yet
    TurbulenceIntensityLateral: float = Field(alias="TurbulenceIntensityLateral", default=None) # Not supported yet
    TurbulenceIntensityVertical: float = Field(alias="TurbulenceIntensityVertical", default=None) # Not supported yet
    CentreTurbulenceFileOn: TurbulentCurrent_CentreTurbulenceFileOnEnum = Field(alias="CentreTurbulenceFileOn", default=None) # Not supported yet
    CentreTurbulenceFileAtHeight: float = Field(alias="CentreTurbulenceFileAtHeight", default=None) # Not supported yet
    RepeatTurbulenceFile: bool = Field(alias="RepeatTurbulenceFile", default=None) # Not supported yet
    TurbulenceFileStartTime: float = Field(alias="TurbulenceFileStartTime", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/Current/TurbulentCurrent.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'CurrentType').merge(TimeDomainCurrent._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


TurbulentCurrent.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class NominalHubWindSpeedVsRotorSpeedAndPitchAngle(BladedModel):
    r"""
    The look-up table for specifying the nominal free-field wind speed at the hub against the turbine's operating condition appropriate for that wind speed.  Wind speeds need to be in monotonic ascending order. Linear interpolation is used between points. The lowest value for the hub wind speed will be taken as the cut-in wind speed for the turbine, and the highest the cut-out wind speed.
    
    Attributes
    ----------
    NominalHubWindSpeed : float
        The nominal free-field wind speed at the hub.
    
    RotorSpeed : float
        The rotor speed to apply for the specified hub wind speed in steady state/initial condition calculations. The generator torque will be calculated to achieve this rotor speed.  If the torque required is negative, a warning will be generated.
    
    PitchAngle : float
        The pitch angle to apply to all blades for the specified hub wind speed in steady state/initial condition calculations.
    
    Notes
    -----
    
    """
    NominalHubWindSpeed: float = Field(alias="NominalHubWindSpeed", default=None)
    RotorSpeed: float = Field(alias="RotorSpeed", default=None)
    PitchAngle: float = Field(alias="PitchAngle", default=None)

    _relative_schema_path = 'Turbine/TurbineOperationalParameters/NominalHubWindSpeedVsRotorSpeedAndPitchAngle/NominalHubWindSpeedVsRotorSpeedAndPitchAngle.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


NominalHubWindSpeedVsRotorSpeedAndPitchAngle.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


class SchemaHelper:
    """
    Helper methods for constructing schema URLs and retrieving the schema version.
    """
    __schema_version__ = "0.7.0"
    __build_version__ = "0.7.0-rc.18+b.71.sha.68d8d40c"
    __Host__ = "https://bladednextgen.dnv.com/schema"

    @staticmethod
    def construct_schema_url(relative_schema_path: str) -> str:
        """
        Constructs a schema URL based on the provided path.

        Parameters
        ----------
        relative_schema_path: str
            The relative path to a schema file within the Bladed schema repository.

        Returns
        -------
        str
            The constructed schema URL.

        """
        return f"{SchemaHelper.__Host__}/{SchemaHelper.__schema_version__}/{relative_schema_path}"


    @staticmethod
    def get_schema_version() -> str:
        """
        Retrieves the schema version this Model API was generated for.
        """
        return SchemaHelper.__schema_version__


# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class TransducerFaults_FaultTypeEnum(str, Enum):
    NONE = "None"
    NOISE = "Noise"
    TO_CONSTANT = "ToConstant"
    CONSTANT_OFFSET = "ConstantOffset"
    GROWING_OFFSET = "GrowingOffset"
    HARMONIC_OFFSET = "HarmonicOffset"

from .schema_helper import SchemaHelper
from .models_impl import *


class TransducerFaults(BladedModel, ABC):
    r"""
    
    
    Not supported yet.
    
    Attributes
    ----------
    ApplyOnlyToFirst : bool, default=True, Not supported yet
        If true, the fault will be aplied only to the first instance of this transducer.  If false, all sensors of this type will fail identically.
    
    MiscalibrationOffset : float, default=0, Not supported yet
        A constant value added to the sensor's output - for example, if the sensor had been incorrectly installed or poorly calibrated.
    
    InverseInstallation : bool, default=False, Not supported yet
        Whether the sensor was installed in reverse, resulting in its outputs being multiplied by -1.0.
    
    FailTime : float, default=0, Not supported yet
        The time at which any time-dependent failure occurs.  This will be from the beginning of the simulation, and not from the start of logging.  Please ensure that this time will equate to a period which is being clearly logged.
    
    FaultType : TransducerFaults_FaultTypeEnum, default='None', Not supported yet
        What fault occurs at a specified time in the simulation.
    
    NoiseMultiplier : float, default=1, Not supported yet
        The factor by which the noise on the signal increases (or decreases) by.
    
    FailureValue : float, default=0, Not supported yet
        The value which the sensor reverts to in the case of failure.
    
    FailureValueRampRate : float, default=0, Not supported yet
        The rate at which the signal reverts to its fail value.  A rate of 0.0 represents an instantaneous step change to the fail value.
    
    ConstantOffset : float, default=0, Not supported yet
        The value by which the signal is offset by.
    
    GrowingOffset : float, default=0, Not supported yet
        The rate at which the signal is deviating from its calibrated value.
    
    Magnitude : float, default=0, Not supported yet
        The greatest magnitude of the harmonic interference from the 'true' value.
    
    Period : float, default=1, Not supported yet
        The time period of the full sinusoidal interference pattern.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    ApplyOnlyToFirst: bool = Field(alias="ApplyOnlyToFirst", default=None) # Not supported yet
    MiscalibrationOffset: float = Field(alias="MiscalibrationOffset", default=None) # Not supported yet
    InverseInstallation: bool = Field(alias="InverseInstallation", default=None) # Not supported yet
    FailTime: float = Field(alias="FailTime", default=None) # Not supported yet
    FaultType: TransducerFaults_FaultTypeEnum = Field(alias="FaultType", default=None) # Not supported yet
    NoiseMultiplier: float = Field(alias="NoiseMultiplier", default=None) # Not supported yet
    FailureValue: float = Field(alias="FailureValue", default=None) # Not supported yet
    FailureValueRampRate: float = Field(alias="FailureValueRampRate", default=None) # Not supported yet
    ConstantOffset: float = Field(alias="ConstantOffset", default=None) # Not supported yet
    GrowingOffset: float = Field(alias="GrowingOffset", default=None) # Not supported yet
    Magnitude: float = Field(alias="Magnitude", default=None) # Not supported yet
    Period: float = Field(alias="Period", default=None) # Not supported yet

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


TransducerFaults.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.pitch_actuation_limited import PitchActuationLimited
from dnv_bladed_models.pitch_constant_rate_safety_system import PitchConstantRateSafetySystem
from dnv_bladed_models.pitch_position_demand import PitchPositionDemand
from dnv_bladed_models.pitch_rate_demand import PitchRateDemand
from dnv_bladed_models.pitch_rate_demand_set_by_external_controller import PitchRateDemandSetByExternalController
from dnv_bladed_models.pitch_rate_varies_with_position import PitchRateVariesWithPosition
from dnv_bladed_models.pitch_rate_varies_with_time import PitchRateVariesWithTime
from dnv_bladed_models.pitch_safety_system import PitchSafetySystem
from dnv_bladed_models.pitch_safety_system_insert import PitchSafetySystemInsert
from dnv_bladed_models.pitch_system_demand import PitchSystemDemand
from dnv_bladed_models.pitch_system_demand_insert import PitchSystemDemandInsert
from dnv_bladed_models.pitch_torque_set_by_external_controller import PitchTorqueSetByExternalController

from .schema_helper import SchemaHelper
from .models_impl import *

TPitchSystemDemandOptions = TypeVar('TPitchSystemDemandOptions', PitchSystemDemandInsert, PitchPositionDemand, PitchRateDemand, PitchSystemDemand, )
TPitchSafetySystemOptions = TypeVar('TPitchSafetySystemOptions', PitchActuationLimited, PitchConstantRateSafetySystem, PitchSafetySystemInsert, PitchRateDemandSetByExternalController, PitchRateVariesWithPosition, PitchRateVariesWithTime, PitchTorqueSetByExternalController, PitchSafetySystem, )

class PitchController(BladedModel):
    r"""
    The definition of the pitch controller.  The pitch system generally has its own independent controller (usually physically separate) which received position or rate demands from the main controller.
    
    Attributes
    ----------
    Demand : Union[PitchSystemDemandInsert, PitchPositionDemand, PitchRateDemand]
    
    SafetySystem : Union[PitchActuationLimited, PitchConstantRateSafetySystem, PitchSafetySystemInsert, PitchRateDemandSetByExternalController, PitchRateVariesWithPosition, PitchRateVariesWithTime, PitchTorqueSetByExternalController]
    
    Notes
    -----
    
    """
    Demand: Union[PitchSystemDemandInsert, PitchPositionDemand, PitchRateDemand] = Field(alias="Demand", default=None, discriminator='PitchSystemDemandType')
    SafetySystem: Union[PitchActuationLimited, PitchConstantRateSafetySystem, PitchSafetySystemInsert, PitchRateDemandSetByExternalController, PitchRateVariesWithPosition, PitchRateVariesWithTime, PitchTorqueSetByExternalController] = Field(alias="SafetySystem", default=None, discriminator='PitchSafetySystemType')

    _relative_schema_path = 'Components/PitchSystem/PitchController/PitchController.json'
    _type_info = TypeInfo(
        set([('Demand', 'PitchSystemDemandType'),('SafetySystem', 'PitchSafetySystemType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def Demand_as_PitchPositionDemand(self) -> PitchPositionDemand:
        """
        Retrieves the value of Demand guaranteeing it is a PitchPositionDemand; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchPositionDemand
            A model object, guaranteed to be a PitchPositionDemand.

        Raises
        ------
        TypeError
            If the value is not a PitchPositionDemand.
        """
        return self.Demand_as(PitchPositionDemand)


    @property
    def Demand_as_PitchRateDemand(self) -> PitchRateDemand:
        """
        Retrieves the value of Demand guaranteeing it is a PitchRateDemand; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchRateDemand
            A model object, guaranteed to be a PitchRateDemand.

        Raises
        ------
        TypeError
            If the value is not a PitchRateDemand.
        """
        return self.Demand_as(PitchRateDemand)


    @property
    def Demand_as_inline(self) -> Union[PitchPositionDemand, PitchRateDemand]:
        """
        Retrieves the value of Demand as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PitchPositionDemand, PitchRateDemand]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of PitchSystemDemand; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Demand, PitchSystemDemandInsert) or self.Demand.is_insert:
            raise TypeError(f"Expected Demand value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Demand


    def Demand_as(self, cls: Type[TPitchSystemDemandOptions])-> TPitchSystemDemandOptions:
        """
        Retrieves the value of Demand, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of PitchSystemDemand, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[PitchSystemDemandInsert, PitchPositionDemand, PitchRateDemand]]
            One of the valid concrete types of PitchSystemDemand, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TPitchSystemDemandOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of PitchSystemDemand:
        >>> val_obj = model_obj.Demand_as(models.PitchPositionDemand)
        >>> val_obj = model_obj.Demand_as(models.PitchRateDemand)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Demand_as(models.PitchSystemDemandInsert)
        """
        if not isinstance(self.Demand, cls):
            raise TypeError(f"Expected Demand of type '{cls.__name__}' but was type '{type(self.Demand).__name__}'")
        return self.Demand


    @property
    def SafetySystem_as_PitchActuationLimited(self) -> PitchActuationLimited:
        """
        Retrieves the value of SafetySystem guaranteeing it is a PitchActuationLimited; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchActuationLimited
            A model object, guaranteed to be a PitchActuationLimited.

        Raises
        ------
        TypeError
            If the value is not a PitchActuationLimited.
        """
        return self.SafetySystem_as(PitchActuationLimited)


    @property
    def SafetySystem_as_PitchConstantRateSafetySystem(self) -> PitchConstantRateSafetySystem:
        """
        Retrieves the value of SafetySystem guaranteeing it is a PitchConstantRateSafetySystem; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchConstantRateSafetySystem
            A model object, guaranteed to be a PitchConstantRateSafetySystem.

        Raises
        ------
        TypeError
            If the value is not a PitchConstantRateSafetySystem.
        """
        return self.SafetySystem_as(PitchConstantRateSafetySystem)


    @property
    def SafetySystem_as_PitchRateDemandSetByExternalController(self) -> PitchRateDemandSetByExternalController:
        """
        Retrieves the value of SafetySystem guaranteeing it is a PitchRateDemandSetByExternalController; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchRateDemandSetByExternalController
            A model object, guaranteed to be a PitchRateDemandSetByExternalController.

        Raises
        ------
        TypeError
            If the value is not a PitchRateDemandSetByExternalController.
        """
        return self.SafetySystem_as(PitchRateDemandSetByExternalController)


    @property
    def SafetySystem_as_PitchRateVariesWithPosition(self) -> PitchRateVariesWithPosition:
        """
        Retrieves the value of SafetySystem guaranteeing it is a PitchRateVariesWithPosition; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchRateVariesWithPosition
            A model object, guaranteed to be a PitchRateVariesWithPosition.

        Raises
        ------
        TypeError
            If the value is not a PitchRateVariesWithPosition.
        """
        return self.SafetySystem_as(PitchRateVariesWithPosition)


    @property
    def SafetySystem_as_PitchRateVariesWithTime(self) -> PitchRateVariesWithTime:
        """
        Retrieves the value of SafetySystem guaranteeing it is a PitchRateVariesWithTime; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchRateVariesWithTime
            A model object, guaranteed to be a PitchRateVariesWithTime.

        Raises
        ------
        TypeError
            If the value is not a PitchRateVariesWithTime.
        """
        return self.SafetySystem_as(PitchRateVariesWithTime)


    @property
    def SafetySystem_as_PitchTorqueSetByExternalController(self) -> PitchTorqueSetByExternalController:
        """
        Retrieves the value of SafetySystem guaranteeing it is a PitchTorqueSetByExternalController; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchTorqueSetByExternalController
            A model object, guaranteed to be a PitchTorqueSetByExternalController.

        Raises
        ------
        TypeError
            If the value is not a PitchTorqueSetByExternalController.
        """
        return self.SafetySystem_as(PitchTorqueSetByExternalController)


    @property
    def SafetySystem_as_inline(self) -> Union[PitchActuationLimited, PitchConstantRateSafetySystem, PitchRateDemandSetByExternalController, PitchRateVariesWithPosition, PitchRateVariesWithTime, PitchTorqueSetByExternalController]:
        """
        Retrieves the value of SafetySystem as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PitchActuationLimited, PitchConstantRateSafetySystem, PitchRateDemandSetByExternalController, PitchRateVariesWithPosition, PitchRateVariesWithTime, PitchTorqueSetByExternalController]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of PitchSafetySystem; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.SafetySystem, PitchSafetySystemInsert) or self.SafetySystem.is_insert:
            raise TypeError(f"Expected SafetySystem value to be an in-line object, but it is currently in the '$insert' state.")
        return self.SafetySystem


    def SafetySystem_as(self, cls: Type[TPitchSafetySystemOptions])-> TPitchSafetySystemOptions:
        """
        Retrieves the value of SafetySystem, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of PitchSafetySystem, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[PitchActuationLimited, PitchConstantRateSafetySystem, PitchSafetySystemInsert, PitchRateDemandSetByExternalController, PitchRateVariesWithPosition, PitchRateVariesWithTime, PitchTorqueSetByExternalController]]
            One of the valid concrete types of PitchSafetySystem, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TPitchSafetySystemOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of PitchSafetySystem:
        >>> val_obj = model_obj.SafetySystem_as(models.PitchActuationLimited)
        >>> val_obj = model_obj.SafetySystem_as(models.PitchConstantRateSafetySystem)
        >>> val_obj = model_obj.SafetySystem_as(models.PitchRateDemandSetByExternalController)
        >>> val_obj = model_obj.SafetySystem_as(models.PitchRateVariesWithPosition)
        >>> val_obj = model_obj.SafetySystem_as(models.PitchRateVariesWithTime)
        >>> val_obj = model_obj.SafetySystem_as(models.PitchTorqueSetByExternalController)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.SafetySystem_as(models.PitchSafetySystemInsert)
        """
        if not isinstance(self.SafetySystem, cls):
            raise TypeError(f"Expected SafetySystem of type '{cls.__name__}' but was type '{type(self.SafetySystem).__name__}'")
        return self.SafetySystem


    def _entity(self) -> bool:
        return True


PitchController.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class EquilibriumStateSolverSettings(BladedModel):
    r"""
    The settings pertaining to the determination of equilibrium conditions, used for steady calculations or initial conditions.
    
    Attributes
    ----------
    ModalStatesRelaxationFactor : float, default=0.4
        The relaxation factor applied to modal states during iterations to find the equilibrium conditions, to improve numerical stability.  This is used to calculate the initial conditions for time domain calculations, and the steady state in steady calculations.
    
    FloatingPlatformStatesRelaxationFactor : float, default=0.4
        The relaxation factor applied to floating platform positional states during iterations to find the equilibrium conditions, to improve numerical stability.  This is used to calculate the initial conditions for time domain calculations, and the steady state in steady calculations.
    
    Notes
    -----
    
    """
    ModalStatesRelaxationFactor: float = Field(alias="ModalStatesRelaxationFactor", default=None)
    FloatingPlatformStatesRelaxationFactor: float = Field(alias="FloatingPlatformStatesRelaxationFactor", default=None)

    _relative_schema_path = 'Settings/SolverSettings/EquilibriumStateSolverSettings/EquilibriumStateSolverSettings.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


EquilibriumStateSolverSettings.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.added_inertia import AddedInertia
from dnv_bladed_models.vector3_d import Vector3D

from .schema_helper import SchemaHelper
from .models_impl import *


class PointInertia(AddedInertia):
    r"""
    An inertia added to a structural component.
    
    Attributes
    ----------
    AddedInertiaType : Literal['PointInertia'], default='PointInertia'
        Defines the specific type of AddedInertia model in use.  For a `PointInertia` object, this must always be set to a value of `PointInertia`.
    
    Mass : float, default=0
        The mass to be added.
    
    Offset : Vector3D
    
    MomentOfInertiaTensor : List[float], default=[0,0,0]
        Moment of inertia tensor acting about the centre of mass. This tensor is defined using the local coordinate system of the attachment node.
    
    CompleteInertiaTensor : float, default=0
        The symmetric inertia tensor acting about the attachment node. This tensor is defined using the local coordinate system of the attachment node.
    
    Notes
    -----
    
    """
    AddedInertiaType: Literal['PointInertia'] = Field(alias="AddedInertiaType", default='PointInertia', allow_mutation=False, const=True) # type: ignore
    Mass: float = Field(alias="Mass", default=None)
    Offset: Vector3D = Field(alias="Offset", default=None)
    MomentOfInertiaTensor: List[float] = Field(alias="MomentOfInertiaTensor", default=list())
    CompleteInertiaTensor: float = Field(alias="CompleteInertiaTensor", default=None)

    _relative_schema_path = 'Components/Tower/AddedInertia/PointInertia.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['MomentOfInertiaTensor',]),
        'AddedInertiaType').merge(AddedInertia._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


PointInertia.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.emergency_stop_button import EmergencyStopButton
from dnv_bladed_models.generator_overpower import GeneratorOverpower
from dnv_bladed_models.generator_overspeed import GeneratorOverspeed
from dnv_bladed_models.generator_short_circuit import GeneratorShortCircuit
from dnv_bladed_models.nacelle_vibration import NacelleVibration
from dnv_bladed_models.rotor_overspeed import RotorOverspeed
from dnv_bladed_models.safety_circuit_library import SafetyCircuitLibrary

from .schema_helper import SchemaHelper
from .models_impl import *


class SafetySystem(BladedModel):
    r"""
    The definition of the turbine's safety system.  This supersedes the turbine's controller requests.
    
    Not supported yet.
    
    Attributes
    ----------
    SafetyCircuitLibrary : SafetyCircuitLibrary, Not supported yet
    
    GeneratorOverspeed : GeneratorOverspeed, Not supported yet
    
    RotorOverspeed : RotorOverspeed, Not supported yet
    
    GeneratorOverpower : GeneratorOverpower, Not supported yet
    
    NacelleVibration : NacelleVibration, Not supported yet
    
    GeneratorShortCircuit : GeneratorShortCircuit, Not supported yet
    
    EmergencyStopButton : EmergencyStopButton, Not supported yet
    
    OnlyActivateOnceLoggingHasCommenced : bool, Not supported yet
        If true, the safety system will be inactivated during the lead in time when there is no logging.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    SafetyCircuitLibrary: SafetyCircuitLibrary = Field(alias="SafetyCircuitLibrary", default=SafetyCircuitLibrary()) # Not supported yet
    GeneratorOverspeed: GeneratorOverspeed = Field(alias="GeneratorOverspeed", default=None) # Not supported yet
    RotorOverspeed: RotorOverspeed = Field(alias="RotorOverspeed", default=None) # Not supported yet
    GeneratorOverpower: GeneratorOverpower = Field(alias="GeneratorOverpower", default=None) # Not supported yet
    NacelleVibration: NacelleVibration = Field(alias="NacelleVibration", default=None) # Not supported yet
    GeneratorShortCircuit: GeneratorShortCircuit = Field(alias="GeneratorShortCircuit", default=None) # Not supported yet
    EmergencyStopButton: EmergencyStopButton = Field(alias="EmergencyStopButton", default=None) # Not supported yet
    OnlyActivateOnceLoggingHasCommenced: bool = Field(alias="OnlyActivateOnceLoggingHasCommenced", default=None) # Not supported yet

    _relative_schema_path = 'Turbine/BladedControl/SafetySystem/SafetySystem.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['SafetyCircuitLibrary',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


SafetySystem.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.hardware_tracking import HardwareTracking

from .schema_helper import SchemaHelper
from .models_impl import *


class ExternallySteppedSimulation(BladedModel):
    r"""
    The definition of a Bladed real time simulation.  This will be run with Bladed as an external library (i.e. a dynamic link library (dll) on Windows, or a shared object (so) on Linux).  An external process must call functions within the Bladed external library to \"step\" the simulation, and thereby keep it synchronised with the external controlling process.
    
    Not supported yet.
    
    Attributes
    ----------
    TimeStep : float, Not supported yet
        The real time simulation time step, as set by the external process that is calling Bladed in real time mode.
    
    TimeStepMultiplier : float, Not supported yet
        The real time simulation time step multiplier.
    
    HardwareTracking : HardwareTracking, Not supported yet
    
    OutputRealTimeDataFromStart : bool, Not supported yet
        If true, realtime data will be output as soon as Bladed is running.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    TimeStep: float = Field(alias="TimeStep", default=None) # Not supported yet
    TimeStepMultiplier: float = Field(alias="TimeStepMultiplier", default=None) # Not supported yet
    HardwareTracking: HardwareTracking = Field(alias="HardwareTracking", default=None) # Not supported yet
    OutputRealTimeDataFromStart: bool = Field(alias="OutputRealTimeDataFromStart", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/ExternallySteppedSimulation/ExternallySteppedSimulation.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ExternallySteppedSimulation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class Event_EventTypeEnum(str, Enum):
    CONTROLLER_FAULT = "ControllerFault"
    EMERGENCY_STOP_OPERATION = "EmergencyStopOperation"
    GRID_LOSS = "GridLoss"
    NETWORK_FREQUENCY_DISTURBANCE = "NetworkFrequencyDisturbance"
    NETWORK_VOLTAGE_DISTURBANCE = "NetworkVoltageDisturbance"
    NORMAL_STOP_OPERATION = "NormalStopOperation"
    PERMANENTLY_STUCK_PITCH_SYSTEM = "PermanentlyStuckPitchSystem"
    PITCH_FAULT_CONSTANT_RATE = "PitchFaultConstantRate"
    PITCH_FAULT_CONSTANT_TORQUE = "PitchFaultConstantTorque"
    PITCH_FAULT_LIMP = "PitchFaultLimp"
    PITCH_FAULT_SEIZURE = "PitchFaultSeizure"
    PITCH_FAULT_SEIZURE_AT_ANGLE = "PitchFaultSeizureAtAngle"
    SHORT_CIRCUIT = "ShortCircuit"
    START_UP_OPERATION = "StartUpOperation"
    YAW_FAULT_CONSTANT_RATE = "YawFaultConstantRate"
    YAW_FAULT_CONSTANT_TORQUE = "YawFaultConstantTorque"
    YAW_FAULT_LIMP = "YawFaultLimp"
    YAW_MANOEUVRE = "YawManoeuvre"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class Event(BladedModel, ABC):
    r"""
    An event which may occur during the simulation.  These can either be timed events, or based on a set of conditions which may or may not occur.
    
    Attributes
    ----------
    EventType : Event_EventTypeEnum
        Defines the specific type of model in use.
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - ControllerFault
        - EmergencyStopOperation
        - GridLoss
        - EventInsert
        - NetworkFrequencyDisturbance
        - NetworkVoltageDisturbance
        - NormalStopOperation
        - PermanentlyStuckPitchSystem
        - PitchFaultConstantRate
        - PitchFaultConstantTorque
        - PitchFaultLimp
        - PitchFaultSeizure
        - PitchFaultSeizureAtAngle
        - ShortCircuit
        - StartUpOperation
        - YawFaultConstantRate
        - YawFaultConstantTorque
        - YawFaultLimp
        - YawManoeuvre
    
    """
    EventType: Event_EventTypeEnum = Field(alias="EventType", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


Event.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class Logging_ConsoleLoggingLevelEnum(str, Enum):
    SILENT = "SILENT"
    EXCEPTIONS = "EXCEPTIONS"
    WARNINGS = "WARNINGS"
    INFORMATION = "INFORMATION"
    DIAGNOSTIC = "DIAGNOSTIC"
class Logging_MessageFileLoggingLevelEnum(str, Enum):
    EXCEPTIONS = "EXCEPTIONS"
    WARNINGS = "WARNINGS"
    INFORMATION = "INFORMATION"
    DIAGNOSTIC = "DIAGNOSTIC"
class Logging_SupportingFilesEnum(str, Enum):
    BASIC = "BASIC"
    VERIFICATION = "VERIFICATION"
    DIAGNOSTIC = "DIAGNOSTIC"
class Logging_VisualisationEnum(str, Enum):
    NONE = "NONE"
    MODEL = "MODEL"
    ANIMATION = "ANIMATION"

from .schema_helper import SchemaHelper
from .models_impl import *


class Logging(BladedModel):
    r"""
    Default settings for the logging of the supporting data, such as verification or progress data.  Many of these settings can be overridden for individual analyses using command line options.
    
    Attributes
    ----------
    ConsoleLoggingLevel : Logging_ConsoleLoggingLevelEnum, default='WARNINGS'
        The level of detail to report to stdout and stderr:   'SILENT': there will be no messages written to the console.   'EXCEPTIONS': only error messages will be written to the console.   'WARNING': error messages plus warning messages.   'INFORMATION': additional notes describing the simulation.   'DIAGNOSTIC': includes additional details which would might help explain more subtle simulation behaviour.
    
    MessageFileLoggingLevel : Logging_MessageFileLoggingLevelEnum, default='INFORMATION'
        The level of detail to report to the $ME message file:   'EXCEPTIONS': only error messages will be written to the file.   'WARNING': error messages plus warning messages.   'INFORMATION': additional notes describing the simulation.   'DIAGNOSTIC': includes additional details which would might help explain more subtle simulation behaviour.
    
    SupportingFiles : Logging_SupportingFilesEnum, default='BASIC'
        The level of supporting files to be written to the output directory during the analysis:   'BASIC' includes .$ME and .$TE files.   'VERIFICATION' produces additional reports summarising the inputs with minimal processing, such as inertia properties and basic mode data.   'DIAGNOSTIC' will output any data during the analysis that may assist in diagnosing problems. This may impair the performance. These options are cumulative, with DIAGNOSTIC including all VERIFICATION outputs; VERIFICATION indluding all CONSOLIDATED_INPUTS; and CONSOLIDATED_INPUTS including all BASIC outputs.
    
    ProgressFile : bool, default=True
        If true, a .$PR file containing a single number detailing the progree will be written to the working directory and updated throughout the analysis.  This can be read by the calling process such as a GUI or batch manager.
    
    Visualisation : Logging_VisualisationEnum, default='NONE'
        The level of additional visualisation files to produce during an analysis:   'MODEL' produces additional files to allow external software to produce a static 3D model of the turbine.   'ANIMATION' produces additional files and time-series to allow external software to animate the movement of the turbine.
    
    Notes
    -----
    
    """
    ConsoleLoggingLevel: Logging_ConsoleLoggingLevelEnum = Field(alias="ConsoleLoggingLevel", default=None)
    MessageFileLoggingLevel: Logging_MessageFileLoggingLevelEnum = Field(alias="MessageFileLoggingLevel", default=None)
    SupportingFiles: Logging_SupportingFilesEnum = Field(alias="SupportingFiles", default=None)
    ProgressFile: bool = Field(alias="ProgressFile", default=None)
    Visualisation: Logging_VisualisationEnum = Field(alias="Visualisation", default=None)

    _relative_schema_path = 'Settings/Logging/Logging.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


Logging.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class NominalHubWindSpeedVsMinimumSteadyStatePitchAngle(BladedModel):
    r"""
    The look-up table for specifying the nominal free-field wind speed at the hub against the respective minimum pitch angle for pitch scheduling. Wind speeds need to be in monotonic ascending order. Linear interpolation is used between points. If the hub wind speed exceeds the specified value the nearest will be used and a warning message will be written.
    
    Attributes
    ----------
    HubWindSpeed : float
        The nominal free-field wind speed at the hub for which the minimum pitch angle will apply.
    
    MinimumSteadyStatePitchAngle : float
        The minimum pitch angle demand of the blades that the turbine control will apply for the specified wind speed.  The initial conditions may choose a higher pitch angle in order to achieve an equilibrium, but not a lower one.
    
    Notes
    -----
    
    """
    HubWindSpeed: float = Field(alias="HubWindSpeed", default=None)
    MinimumSteadyStatePitchAngle: float = Field(alias="MinimumSteadyStatePitchAngle", default=None)

    _relative_schema_path = 'Turbine/TurbineOperationalParameters/VariableSpeedPitchRegulatedControlModel/NominalHubWindSpeedVsMinimumSteadyStatePitchAngle/NominalHubWindSpeedVsMinimumSteadyStatePitchAngle.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


NominalHubWindSpeedVsMinimumSteadyStatePitchAngle.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.vector3_d import Vector3D
class ConnectableNode_ComponentTypesAllowedEnum(str, Enum):
    COMPONENTS = "COMPONENTS"

from .schema_helper import SchemaHelper
from .models_impl import *


class ConnectableNode(BladedModel, ABC):
    r"""
    An object declaring a distal node for a component.
    
    Not supported yet.
    
    Attributes
    ----------
    ComponentTypesAllowed : ConnectableNode_ComponentTypesAllowedEnum, Not supported yet
        Optional: A list of the components that could be attached to the node.  If not specified, this will be governed by the component's connectivity.
    
    RelativeOrientation : List[float], Not supported yet
        The relative orientation of the attached component to the current component (e.g. the axle to the nacelle, or the blade assembly to the hub).  Required if encrypted and not the same as the parent (e.g. for a nacelle's axle node)
    
    OffsetIfEncrypted : Vector3D
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    ComponentTypesAllowed: ConnectableNode_ComponentTypesAllowedEnum = Field(alias="ComponentTypesAllowed", default=None) # Not supported yet
    RelativeOrientation: List[float] = Field(alias="RelativeOrientation", default=list()) # Not supported yet
    OffsetIfEncrypted: Vector3D = Field(alias="OffsetIfEncrypted", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['RelativeOrientation',]),
        None).merge(BladedModel._type_info)


ConnectableNode.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class SafetySystemCircuit(BladedModel):
    r"""
    A safety system circuit, which defines the actions that will be taken if it is triggered.
    
    Not supported yet.
    
    Attributes
    ----------
    PitchAction : bool, Not supported yet
        If true, the pitch system will be asked to activate its own safety system.
    
    ApplyShaftBrake1 : bool, Not supported yet
        If true, the third shaft brake will be activated.
    
    ApplyShaftBrake2 : bool, Not supported yet
        If true, the third shaft brake will be activated.
    
    ApplyShaftBrake3 : bool, Not supported yet
        If true, the third shaft brake will be activated.
    
    DisconnectGenerator : bool, Not supported yet
        If true, the generator will be disconnected from the grid.
    
    ApplyGeneratorBrake : bool, Not supported yet
        If true, the generator brake will be activated.
    
    DisconnectYawDrive : bool, Not supported yet
        If true, the yaw drive will be disconnected.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    PitchAction: bool = Field(alias="PitchAction", default=None) # Not supported yet
    ApplyShaftBrake1: bool = Field(alias="ApplyShaftBrake1", default=None) # Not supported yet
    ApplyShaftBrake2: bool = Field(alias="ApplyShaftBrake2", default=None) # Not supported yet
    ApplyShaftBrake3: bool = Field(alias="ApplyShaftBrake3", default=None) # Not supported yet
    DisconnectGenerator: bool = Field(alias="DisconnectGenerator", default=None) # Not supported yet
    ApplyGeneratorBrake: bool = Field(alias="ApplyGeneratorBrake", default=None) # Not supported yet
    DisconnectYawDrive: bool = Field(alias="DisconnectYawDrive", default=None) # Not supported yet

    _relative_schema_path = 'Turbine/BladedControl/SafetySystem/SafetyCircuitLibrary/SafetySystemCircuit/SafetySystemCircuit.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


SafetySystemCircuit.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.tower_can_cross_section_with_sensors import TowerCanCrossSectionWithSensors

from .schema_helper import SchemaHelper
from .models_impl import *


class TowerCanCrossSectionPropertiesWhereDifferent(TowerCanCrossSectionWithSensors, ABC):
    r"""
    The definition of a can cross-section where the material properties, the diameter, and the wall thickness will be used to calculate the structural properties.  Any properties which are omitted will be taken from the BaseCrossSection definition.
    
    Attributes
    ----------
    OutsideDiameter : float
        The external diameter of the can cross-section.  If omitted, the value from the BaseCrossSection will be used.
    
    Notes
    -----
    
    """
    OutsideDiameter: float = Field(alias="OutsideDiameter", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(TowerCanCrossSectionWithSensors._type_info)


TowerCanCrossSectionPropertiesWhereDifferent.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.pitch_controller_rate_safety_system import PitchControllerRateSafetySystem
from dnv_bladed_models.rate_vs_time import RateVsTime

from .schema_helper import SchemaHelper
from .models_impl import *


class PitchRateVariesWithTime(PitchControllerRateSafetySystem):
    r"""
    A safety system where the pitch system will adjust its rate dependent on the time after which the safety system has been triggered.
    
    Attributes
    ----------
    PitchSafetySystemType : Literal['RateVariesWithTime'], default='RateVariesWithTime'
        Defines the specific type of PitchSafetySystem model in use.  For a `RateVariesWithTime` object, this must always be set to a value of `RateVariesWithTime`.
    
    TimeVsRate : List[RateVsTime]
        A look-up table of the rate versus the time after the safety system has been triggered.
    
    Notes
    -----
    
    """
    PitchSafetySystemType: Literal['RateVariesWithTime'] = Field(alias="PitchSafetySystemType", default='RateVariesWithTime', allow_mutation=False, const=True) # type: ignore
    TimeVsRate: List[RateVsTime] = Field(alias="TimeVsRate", default=list())

    _relative_schema_path = 'Components/PitchSystem/PitchController/PitchSafetySystem/PitchRateVariesWithTime.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['TimeVsRate',]),
        'PitchSafetySystemType').merge(PitchControllerRateSafetySystem._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


PitchRateVariesWithTime.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.vector3_d import Vector3D
from dnv_bladed_models.waves import Waves
class WaveSpectrum_WaveCurrentInteractionMethodEnum(str, Enum):
    TYPE1 = "Type1"
    TYPE2A = "Type2a"
    TYPE2B = "Type2b"
    TYPE2C_I = "Type2c_i"
    TYPE2C_II = "Type2c_ii"

from .schema_helper import SchemaHelper
from .models_impl import *


class WaveSpectrum(Waves):
    r"""
    
    
    Not supported yet.
    
    Attributes
    ----------
    WavesType : Literal['WaveSpectrum'], default='WaveSpectrum', Not supported yet
        Defines the specific type of Waves model in use.  For a `WaveSpectrum` object, this must always be set to a value of `WaveSpectrum`.
    
    PhaseOrigin : Vector3D
    
    SeaStateFilepath : str, Not supported yet
        The location of .SEA file describing the simulation seastate.
    
    WaveCurrentInteractionMethod : WaveSpectrum_WaveCurrentInteractionMethodEnum, default='Type1', Not supported yet
        The method to be used in combining waves and currents to obtain overall water particle kinematics.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    WavesType: Literal['WaveSpectrum'] = Field(alias="WavesType", default='WaveSpectrum', allow_mutation=False, const=True) # Not supported yet # type: ignore
    PhaseOrigin: Vector3D = Field(alias="PhaseOrigin", default=None)
    SeaStateFilepath: str = Field(alias="SeaStateFilepath", default=None) # Not supported yet
    WaveCurrentInteractionMethod: WaveSpectrum_WaveCurrentInteractionMethodEnum = Field(alias="WaveCurrentInteractionMethod", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/Waves/WaveSpectrum.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'WavesType').merge(Waves._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


WaveSpectrum.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.current_direction_shear_time_history import CurrentDirectionShearTimeHistory
from dnv_bladed_models.current_direction_shear_variation import CurrentDirectionShearVariation
from dnv_bladed_models.current_direction_shear_variation_insert import CurrentDirectionShearVariationInsert
from dnv_bladed_models.current_horizontal_shear_time_history import CurrentHorizontalShearTimeHistory
from dnv_bladed_models.current_horizontal_shear_variation import CurrentHorizontalShearVariation
from dnv_bladed_models.current_horizontal_shear_variation_insert import CurrentHorizontalShearVariationInsert
from dnv_bladed_models.current_mean_speed_time_history import CurrentMeanSpeedTimeHistory
from dnv_bladed_models.current_mean_speed_variation import CurrentMeanSpeedVariation
from dnv_bladed_models.current_mean_speed_variation_insert import CurrentMeanSpeedVariationInsert
from dnv_bladed_models.current_vertical_shear_time_history import CurrentVerticalShearTimeHistory
from dnv_bladed_models.current_vertical_shear_variation import CurrentVerticalShearVariation
from dnv_bladed_models.current_vertical_shear_variation_insert import CurrentVerticalShearVariationInsert
from dnv_bladed_models.preset_current_direction_shear_transient import PresetCurrentDirectionShearTransient
from dnv_bladed_models.preset_current_horizontal_shear_transient import PresetCurrentHorizontalShearTransient
from dnv_bladed_models.preset_current_mean_speed_transient import PresetCurrentMeanSpeedTransient
from dnv_bladed_models.preset_current_vertical_shear_transient import PresetCurrentVerticalShearTransient
from dnv_bladed_models.time_domain_current import TimeDomainCurrent

from .schema_helper import SchemaHelper
from .models_impl import *

TCurrentMeanSpeedVariationOptions = TypeVar('TCurrentMeanSpeedVariationOptions', CurrentMeanSpeedVariationInsert, PresetCurrentMeanSpeedTransient, CurrentMeanSpeedTimeHistory, CurrentMeanSpeedVariation, )
TCurrentVerticalShearVariationOptions = TypeVar('TCurrentVerticalShearVariationOptions', CurrentVerticalShearVariationInsert, PresetCurrentVerticalShearTransient, CurrentVerticalShearTimeHistory, CurrentVerticalShearVariation, )
TCurrentHorizontalShearVariationOptions = TypeVar('TCurrentHorizontalShearVariationOptions', CurrentHorizontalShearVariationInsert, PresetCurrentHorizontalShearTransient, CurrentHorizontalShearTimeHistory, CurrentHorizontalShearVariation, )
TCurrentDirectionShearVariationOptions = TypeVar('TCurrentDirectionShearVariationOptions', CurrentDirectionShearVariationInsert, PresetCurrentDirectionShearTransient, CurrentDirectionShearTimeHistory, CurrentDirectionShearVariation, )

class LaminarFlowCurrent(TimeDomainCurrent):
    r"""
    The definition of a current that varies throughout a time domain simulation, but does not have turbulence.
    
    Not supported yet.
    
    Attributes
    ----------
    CurrentType : Literal['LaminarFlow'], default='LaminarFlow', Not supported yet
        Defines the specific type of Current model in use.  For a `LaminarFlow` object, this must always be set to a value of `LaminarFlow`.
    
    MeanSpeed : float, Not supported yet
        The (constant) mean speed of the current for the duration of the simulation.
    
    MeanSpeedVariation : Union[CurrentMeanSpeedVariationInsert, PresetCurrentMeanSpeedTransient, CurrentMeanSpeedTimeHistory], Not supported yet
    
    VerticalShearVariation : Union[CurrentVerticalShearVariationInsert, PresetCurrentVerticalShearTransient, CurrentVerticalShearTimeHistory], Not supported yet
    
    HorizontalShearVariation : Union[CurrentHorizontalShearVariationInsert, PresetCurrentHorizontalShearTransient, CurrentHorizontalShearTimeHistory], Not supported yet
    
    DirectionShearVariation : Union[CurrentDirectionShearVariationInsert, PresetCurrentDirectionShearTransient, CurrentDirectionShearTimeHistory], Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    CurrentType: Literal['LaminarFlow'] = Field(alias="CurrentType", default='LaminarFlow', allow_mutation=False, const=True) # Not supported yet # type: ignore
    MeanSpeed: float = Field(alias="MeanSpeed", default=None) # Not supported yet
    MeanSpeedVariation: Union[CurrentMeanSpeedVariationInsert, PresetCurrentMeanSpeedTransient, CurrentMeanSpeedTimeHistory] = Field(alias="MeanSpeedVariation", default=None, discriminator='MeanSpeedVariationType') # Not supported yet
    VerticalShearVariation: Union[CurrentVerticalShearVariationInsert, PresetCurrentVerticalShearTransient, CurrentVerticalShearTimeHistory] = Field(alias="VerticalShearVariation", default=None, discriminator='VerticalShearVariationType') # Not supported yet
    HorizontalShearVariation: Union[CurrentHorizontalShearVariationInsert, PresetCurrentHorizontalShearTransient, CurrentHorizontalShearTimeHistory] = Field(alias="HorizontalShearVariation", default=None, discriminator='HorizontalShearVariationType') # Not supported yet
    DirectionShearVariation: Union[CurrentDirectionShearVariationInsert, PresetCurrentDirectionShearTransient, CurrentDirectionShearTimeHistory] = Field(alias="DirectionShearVariation", default=None, discriminator='DirectionShearVariationType') # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/Current/LaminarFlowCurrent.json'
    _type_info = TypeInfo(
        set([('MeanSpeedVariation', 'MeanSpeedVariationType'),('VerticalShearVariation', 'VerticalShearVariationType'),('HorizontalShearVariation', 'HorizontalShearVariationType'),('DirectionShearVariation', 'DirectionShearVariationType'),]),
        set([]),
        set([]),
        'CurrentType').merge(TimeDomainCurrent._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def MeanSpeedVariation_as_PresetCurrentMeanSpeedTransient(self) -> PresetCurrentMeanSpeedTransient:
        """
        Retrieves the value of MeanSpeedVariation guaranteeing it is a PresetCurrentMeanSpeedTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetCurrentMeanSpeedTransient
            A model object, guaranteed to be a PresetCurrentMeanSpeedTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetCurrentMeanSpeedTransient.
        """
        return self.MeanSpeedVariation_as(PresetCurrentMeanSpeedTransient)


    @property
    def MeanSpeedVariation_as_CurrentMeanSpeedTimeHistory(self) -> CurrentMeanSpeedTimeHistory:
        """
        Retrieves the value of MeanSpeedVariation guaranteeing it is a CurrentMeanSpeedTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        CurrentMeanSpeedTimeHistory
            A model object, guaranteed to be a CurrentMeanSpeedTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a CurrentMeanSpeedTimeHistory.
        """
        return self.MeanSpeedVariation_as(CurrentMeanSpeedTimeHistory)


    @property
    def MeanSpeedVariation_as_inline(self) -> Union[PresetCurrentMeanSpeedTransient, CurrentMeanSpeedTimeHistory]:
        """
        Retrieves the value of MeanSpeedVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetCurrentMeanSpeedTransient, CurrentMeanSpeedTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of CurrentMeanSpeedVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.MeanSpeedVariation, CurrentMeanSpeedVariationInsert) or self.MeanSpeedVariation.is_insert:
            raise TypeError(f"Expected MeanSpeedVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.MeanSpeedVariation


    def MeanSpeedVariation_as(self, cls: Type[TCurrentMeanSpeedVariationOptions])-> TCurrentMeanSpeedVariationOptions:
        """
        Retrieves the value of MeanSpeedVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of CurrentMeanSpeedVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[CurrentMeanSpeedVariationInsert, PresetCurrentMeanSpeedTransient, CurrentMeanSpeedTimeHistory]]
            One of the valid concrete types of CurrentMeanSpeedVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TCurrentMeanSpeedVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of CurrentMeanSpeedVariation:
        >>> val_obj = model_obj.MeanSpeedVariation_as(models.PresetCurrentMeanSpeedTransient)
        >>> val_obj = model_obj.MeanSpeedVariation_as(models.CurrentMeanSpeedTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.MeanSpeedVariation_as(models.CurrentMeanSpeedVariationInsert)
        """
        if not isinstance(self.MeanSpeedVariation, cls):
            raise TypeError(f"Expected MeanSpeedVariation of type '{cls.__name__}' but was type '{type(self.MeanSpeedVariation).__name__}'")
        return self.MeanSpeedVariation


    @property
    def VerticalShearVariation_as_PresetCurrentVerticalShearTransient(self) -> PresetCurrentVerticalShearTransient:
        """
        Retrieves the value of VerticalShearVariation guaranteeing it is a PresetCurrentVerticalShearTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetCurrentVerticalShearTransient
            A model object, guaranteed to be a PresetCurrentVerticalShearTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetCurrentVerticalShearTransient.
        """
        return self.VerticalShearVariation_as(PresetCurrentVerticalShearTransient)


    @property
    def VerticalShearVariation_as_CurrentVerticalShearTimeHistory(self) -> CurrentVerticalShearTimeHistory:
        """
        Retrieves the value of VerticalShearVariation guaranteeing it is a CurrentVerticalShearTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        CurrentVerticalShearTimeHistory
            A model object, guaranteed to be a CurrentVerticalShearTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a CurrentVerticalShearTimeHistory.
        """
        return self.VerticalShearVariation_as(CurrentVerticalShearTimeHistory)


    @property
    def VerticalShearVariation_as_inline(self) -> Union[PresetCurrentVerticalShearTransient, CurrentVerticalShearTimeHistory]:
        """
        Retrieves the value of VerticalShearVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetCurrentVerticalShearTransient, CurrentVerticalShearTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of CurrentVerticalShearVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.VerticalShearVariation, CurrentVerticalShearVariationInsert) or self.VerticalShearVariation.is_insert:
            raise TypeError(f"Expected VerticalShearVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.VerticalShearVariation


    def VerticalShearVariation_as(self, cls: Type[TCurrentVerticalShearVariationOptions])-> TCurrentVerticalShearVariationOptions:
        """
        Retrieves the value of VerticalShearVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of CurrentVerticalShearVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[CurrentVerticalShearVariationInsert, PresetCurrentVerticalShearTransient, CurrentVerticalShearTimeHistory]]
            One of the valid concrete types of CurrentVerticalShearVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TCurrentVerticalShearVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of CurrentVerticalShearVariation:
        >>> val_obj = model_obj.VerticalShearVariation_as(models.PresetCurrentVerticalShearTransient)
        >>> val_obj = model_obj.VerticalShearVariation_as(models.CurrentVerticalShearTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.VerticalShearVariation_as(models.CurrentVerticalShearVariationInsert)
        """
        if not isinstance(self.VerticalShearVariation, cls):
            raise TypeError(f"Expected VerticalShearVariation of type '{cls.__name__}' but was type '{type(self.VerticalShearVariation).__name__}'")
        return self.VerticalShearVariation


    @property
    def HorizontalShearVariation_as_PresetCurrentHorizontalShearTransient(self) -> PresetCurrentHorizontalShearTransient:
        """
        Retrieves the value of HorizontalShearVariation guaranteeing it is a PresetCurrentHorizontalShearTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetCurrentHorizontalShearTransient
            A model object, guaranteed to be a PresetCurrentHorizontalShearTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetCurrentHorizontalShearTransient.
        """
        return self.HorizontalShearVariation_as(PresetCurrentHorizontalShearTransient)


    @property
    def HorizontalShearVariation_as_CurrentHorizontalShearTimeHistory(self) -> CurrentHorizontalShearTimeHistory:
        """
        Retrieves the value of HorizontalShearVariation guaranteeing it is a CurrentHorizontalShearTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        CurrentHorizontalShearTimeHistory
            A model object, guaranteed to be a CurrentHorizontalShearTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a CurrentHorizontalShearTimeHistory.
        """
        return self.HorizontalShearVariation_as(CurrentHorizontalShearTimeHistory)


    @property
    def HorizontalShearVariation_as_inline(self) -> Union[PresetCurrentHorizontalShearTransient, CurrentHorizontalShearTimeHistory]:
        """
        Retrieves the value of HorizontalShearVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetCurrentHorizontalShearTransient, CurrentHorizontalShearTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of CurrentHorizontalShearVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.HorizontalShearVariation, CurrentHorizontalShearVariationInsert) or self.HorizontalShearVariation.is_insert:
            raise TypeError(f"Expected HorizontalShearVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.HorizontalShearVariation


    def HorizontalShearVariation_as(self, cls: Type[TCurrentHorizontalShearVariationOptions])-> TCurrentHorizontalShearVariationOptions:
        """
        Retrieves the value of HorizontalShearVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of CurrentHorizontalShearVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[CurrentHorizontalShearVariationInsert, PresetCurrentHorizontalShearTransient, CurrentHorizontalShearTimeHistory]]
            One of the valid concrete types of CurrentHorizontalShearVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TCurrentHorizontalShearVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of CurrentHorizontalShearVariation:
        >>> val_obj = model_obj.HorizontalShearVariation_as(models.PresetCurrentHorizontalShearTransient)
        >>> val_obj = model_obj.HorizontalShearVariation_as(models.CurrentHorizontalShearTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.HorizontalShearVariation_as(models.CurrentHorizontalShearVariationInsert)
        """
        if not isinstance(self.HorizontalShearVariation, cls):
            raise TypeError(f"Expected HorizontalShearVariation of type '{cls.__name__}' but was type '{type(self.HorizontalShearVariation).__name__}'")
        return self.HorizontalShearVariation


    @property
    def DirectionShearVariation_as_PresetCurrentDirectionShearTransient(self) -> PresetCurrentDirectionShearTransient:
        """
        Retrieves the value of DirectionShearVariation guaranteeing it is a PresetCurrentDirectionShearTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetCurrentDirectionShearTransient
            A model object, guaranteed to be a PresetCurrentDirectionShearTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetCurrentDirectionShearTransient.
        """
        return self.DirectionShearVariation_as(PresetCurrentDirectionShearTransient)


    @property
    def DirectionShearVariation_as_CurrentDirectionShearTimeHistory(self) -> CurrentDirectionShearTimeHistory:
        """
        Retrieves the value of DirectionShearVariation guaranteeing it is a CurrentDirectionShearTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        CurrentDirectionShearTimeHistory
            A model object, guaranteed to be a CurrentDirectionShearTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a CurrentDirectionShearTimeHistory.
        """
        return self.DirectionShearVariation_as(CurrentDirectionShearTimeHistory)


    @property
    def DirectionShearVariation_as_inline(self) -> Union[PresetCurrentDirectionShearTransient, CurrentDirectionShearTimeHistory]:
        """
        Retrieves the value of DirectionShearVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetCurrentDirectionShearTransient, CurrentDirectionShearTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of CurrentDirectionShearVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.DirectionShearVariation, CurrentDirectionShearVariationInsert) or self.DirectionShearVariation.is_insert:
            raise TypeError(f"Expected DirectionShearVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.DirectionShearVariation


    def DirectionShearVariation_as(self, cls: Type[TCurrentDirectionShearVariationOptions])-> TCurrentDirectionShearVariationOptions:
        """
        Retrieves the value of DirectionShearVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of CurrentDirectionShearVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[CurrentDirectionShearVariationInsert, PresetCurrentDirectionShearTransient, CurrentDirectionShearTimeHistory]]
            One of the valid concrete types of CurrentDirectionShearVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TCurrentDirectionShearVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of CurrentDirectionShearVariation:
        >>> val_obj = model_obj.DirectionShearVariation_as(models.PresetCurrentDirectionShearTransient)
        >>> val_obj = model_obj.DirectionShearVariation_as(models.CurrentDirectionShearTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.DirectionShearVariation_as(models.CurrentDirectionShearVariationInsert)
        """
        if not isinstance(self.DirectionShearVariation, cls):
            raise TypeError(f"Expected DirectionShearVariation of type '{cls.__name__}' but was type '{type(self.DirectionShearVariation).__name__}'")
        return self.DirectionShearVariation


    def _entity(self) -> bool:
        return True


LaminarFlowCurrent.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.radius_vs_deficit import RadiusVsDeficit
from dnv_bladed_models.steady_wake_deficit import SteadyWakeDeficit

from .schema_helper import SchemaHelper
from .models_impl import *


class UserDefinedWakeDeficit(SteadyWakeDeficit):
    r"""
    A simple model for the reduced velocity of the wind behind another turbine.  This deficit will be applied across a certain region, and this region will not move around during the simulation.  The velocity deficit profile is a look-up table of the radius against the deficit at that radius.
    
    Attributes
    ----------
    SteadyWakeDeficitType : Literal['UserDefined'], default='UserDefined'
        Defines the specific type of SteadyWakeDeficit model in use.  For a `UserDefined` object, this must always be set to a value of `UserDefined`.
    
    RadiusVsDeficit : List[RadiusVsDeficit]
        A list of radii vs the deficit at that radius.  The first value should be at a radius of 0.0, and the largest radius will be the extent of the affected region.  At this point, the deficit is usually 0.0.
    
    Notes
    -----
    
    """
    SteadyWakeDeficitType: Literal['UserDefined'] = Field(alias="SteadyWakeDeficitType", default='UserDefined', allow_mutation=False, const=True) # type: ignore
    RadiusVsDeficit: List[RadiusVsDeficit] = Field(alias="RadiusVsDeficit", default=list())

    _relative_schema_path = 'TimeDomainSimulation/Environment/Wind/SteadyWakeDeficit/UserDefinedWakeDeficit.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['RadiusVsDeficit',]),
        'SteadyWakeDeficitType').merge(SteadyWakeDeficit._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


UserDefinedWakeDeficit.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.aerodynamic_model import AerodynamicModel
from dnv_bladed_models.aerodynamic_model_insert import AerodynamicModelInsert
from dnv_bladed_models.blade_element_momentum import BladeElementMomentum
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.compressible_beddoes_leishman_model import CompressibleBeddoesLeishmanModel
from dnv_bladed_models.dynamic_stall import DynamicStall
from dnv_bladed_models.dynamic_stall_insert import DynamicStallInsert
from dnv_bladed_models.iag_model import IAGModel
from dnv_bladed_models.incompressible_beddoes_leishman_model import IncompressibleBeddoesLeishmanModel
from dnv_bladed_models.oye_model import OyeModel
from dnv_bladed_models.vortex_line import VortexLine

from .schema_helper import SchemaHelper
from .models_impl import *

TAerodynamicModelOptions = TypeVar('TAerodynamicModelOptions', BladeElementMomentum, AerodynamicModelInsert, VortexLine, AerodynamicModel, )
TDynamicStallOptions = TypeVar('TDynamicStallOptions', CompressibleBeddoesLeishmanModel, IAGModel, IncompressibleBeddoesLeishmanModel, DynamicStallInsert, OyeModel, DynamicStall, )

class AerodynamicSettings(BladedModel):
    r"""
    Settings controlling the aerodynamic modelling in Bladed.
    
    Attributes
    ----------
    AerodynamicModel : Union[BladeElementMomentum, AerodynamicModelInsert, VortexLine]
    
    DynamicStall : Union[CompressibleBeddoesLeishmanModel, IAGModel, IncompressibleBeddoesLeishmanModel, DynamicStallInsert, OyeModel]
    
    Notes
    -----
    
    """
    AerodynamicModel: Union[BladeElementMomentum, AerodynamicModelInsert, VortexLine] = Field(alias="AerodynamicModel", default=None, discriminator='AerodynamicModelType')
    DynamicStall: Union[CompressibleBeddoesLeishmanModel, IAGModel, IncompressibleBeddoesLeishmanModel, DynamicStallInsert, OyeModel] = Field(alias="DynamicStall", default=None, discriminator='DynamicStallType')

    _relative_schema_path = 'Settings/AerodynamicSettings/AerodynamicSettings.json'
    _type_info = TypeInfo(
        set([('AerodynamicModel', 'AerodynamicModelType'),('DynamicStall', 'DynamicStallType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def AerodynamicModel_as_BladeElementMomentum(self) -> BladeElementMomentum:
        """
        Retrieves the value of AerodynamicModel guaranteeing it is a BladeElementMomentum; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        BladeElementMomentum
            A model object, guaranteed to be a BladeElementMomentum.

        Raises
        ------
        TypeError
            If the value is not a BladeElementMomentum.
        """
        return self.AerodynamicModel_as(BladeElementMomentum)


    @property
    def AerodynamicModel_as_VortexLine(self) -> VortexLine:
        """
        Retrieves the value of AerodynamicModel guaranteeing it is a VortexLine; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        VortexLine
            A model object, guaranteed to be a VortexLine.

        Raises
        ------
        TypeError
            If the value is not a VortexLine.
        """
        return self.AerodynamicModel_as(VortexLine)


    @property
    def AerodynamicModel_as_inline(self) -> Union[BladeElementMomentum, VortexLine]:
        """
        Retrieves the value of AerodynamicModel as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[BladeElementMomentum, VortexLine]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of AerodynamicModel; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.AerodynamicModel, AerodynamicModelInsert) or self.AerodynamicModel.is_insert:
            raise TypeError(f"Expected AerodynamicModel value to be an in-line object, but it is currently in the '$insert' state.")
        return self.AerodynamicModel


    def AerodynamicModel_as(self, cls: Type[TAerodynamicModelOptions])-> TAerodynamicModelOptions:
        """
        Retrieves the value of AerodynamicModel, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of AerodynamicModel, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[BladeElementMomentum, AerodynamicModelInsert, VortexLine]]
            One of the valid concrete types of AerodynamicModel, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TAerodynamicModelOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of AerodynamicModel:
        >>> val_obj = model_obj.AerodynamicModel_as(models.BladeElementMomentum)
        >>> val_obj = model_obj.AerodynamicModel_as(models.VortexLine)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.AerodynamicModel_as(models.AerodynamicModelInsert)
        """
        if not isinstance(self.AerodynamicModel, cls):
            raise TypeError(f"Expected AerodynamicModel of type '{cls.__name__}' but was type '{type(self.AerodynamicModel).__name__}'")
        return self.AerodynamicModel


    @property
    def DynamicStall_as_CompressibleBeddoesLeishmanModel(self) -> CompressibleBeddoesLeishmanModel:
        """
        Retrieves the value of DynamicStall guaranteeing it is a CompressibleBeddoesLeishmanModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        CompressibleBeddoesLeishmanModel
            A model object, guaranteed to be a CompressibleBeddoesLeishmanModel.

        Raises
        ------
        TypeError
            If the value is not a CompressibleBeddoesLeishmanModel.
        """
        return self.DynamicStall_as(CompressibleBeddoesLeishmanModel)


    @property
    def DynamicStall_as_IAGModel(self) -> IAGModel:
        """
        Retrieves the value of DynamicStall guaranteeing it is a IAGModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        IAGModel
            A model object, guaranteed to be a IAGModel.

        Raises
        ------
        TypeError
            If the value is not a IAGModel.
        """
        return self.DynamicStall_as(IAGModel)


    @property
    def DynamicStall_as_IncompressibleBeddoesLeishmanModel(self) -> IncompressibleBeddoesLeishmanModel:
        """
        Retrieves the value of DynamicStall guaranteeing it is a IncompressibleBeddoesLeishmanModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        IncompressibleBeddoesLeishmanModel
            A model object, guaranteed to be a IncompressibleBeddoesLeishmanModel.

        Raises
        ------
        TypeError
            If the value is not a IncompressibleBeddoesLeishmanModel.
        """
        return self.DynamicStall_as(IncompressibleBeddoesLeishmanModel)


    @property
    def DynamicStall_as_OyeModel(self) -> OyeModel:
        """
        Retrieves the value of DynamicStall guaranteeing it is a OyeModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        OyeModel
            A model object, guaranteed to be a OyeModel.

        Raises
        ------
        TypeError
            If the value is not a OyeModel.
        """
        return self.DynamicStall_as(OyeModel)


    @property
    def DynamicStall_as_inline(self) -> Union[CompressibleBeddoesLeishmanModel, IAGModel, IncompressibleBeddoesLeishmanModel, OyeModel]:
        """
        Retrieves the value of DynamicStall as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[CompressibleBeddoesLeishmanModel, IAGModel, IncompressibleBeddoesLeishmanModel, OyeModel]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of DynamicStall; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.DynamicStall, DynamicStallInsert) or self.DynamicStall.is_insert:
            raise TypeError(f"Expected DynamicStall value to be an in-line object, but it is currently in the '$insert' state.")
        return self.DynamicStall


    def DynamicStall_as(self, cls: Type[TDynamicStallOptions])-> TDynamicStallOptions:
        """
        Retrieves the value of DynamicStall, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of DynamicStall, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[CompressibleBeddoesLeishmanModel, IAGModel, IncompressibleBeddoesLeishmanModel, DynamicStallInsert, OyeModel]]
            One of the valid concrete types of DynamicStall, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TDynamicStallOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of DynamicStall:
        >>> val_obj = model_obj.DynamicStall_as(models.CompressibleBeddoesLeishmanModel)
        >>> val_obj = model_obj.DynamicStall_as(models.IAGModel)
        >>> val_obj = model_obj.DynamicStall_as(models.IncompressibleBeddoesLeishmanModel)
        >>> val_obj = model_obj.DynamicStall_as(models.OyeModel)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.DynamicStall_as(models.DynamicStallInsert)
        """
        if not isinstance(self.DynamicStall, cls):
            raise TypeError(f"Expected DynamicStall of type '{cls.__name__}' but was type '{type(self.DynamicStall).__name__}'")
        return self.DynamicStall


    def _entity(self) -> bool:
        return True


AerodynamicSettings.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class WakePropertiesOfUpstreamTurbine(BladedModel):
    r"""
    
    
    Attributes
    ----------
    AerodynamicInformationResultsPath : str
        A file or URI containing aerodynamic information results, used to generate the initial wake profile of the upwind turbine
    
    SteadyOperationalLoadsResultsPath : str
        A file or URI containing aerodynamic information outputs from this 'Steady Operational Loads' run generates the initial wake profile of the upwind turbine. The steady operational loads run should be performed over all wind speeds and have aerodynamic information outputs switched on for blade 1 at all blade stations.
    
    Notes
    -----
    
    """
    AerodynamicInformationResultsPath: str = Field(alias="AerodynamicInformationResultsPath", default=None)
    SteadyOperationalLoadsResultsPath: str = Field(alias="SteadyOperationalLoadsResultsPath", default=None)

    _relative_schema_path = 'TimeDomainSimulation/Environment/Wind/DynamicUpstreamWake/WakePropertiesOfUpstreamTurbine/WakePropertiesOfUpstreamTurbine.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


WakePropertiesOfUpstreamTurbine.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class StructuralModelling_StructuralModellingTypeEnum(str, Enum):
    MODAL_STRUCTURAL_MODELLING = "ModalStructuralModelling"
    RIGID_STRUCTURAL_MODELLING = "RigidStructuralModelling"
    INSERT = "Insert"
class StructuralModelling_GeometricStiffnessModelEnum(str, Enum):
    AXIAL_LOADS_ONLY = "AxialLoadsOnly"
    INTERNAL_LOADS_ONLY = "InternalLoadsOnly"
    DISABLED = "Disabled"

from .schema_helper import SchemaHelper
from .models_impl import *


class StructuralModelling(BladedModel, ABC):
    r"""
    The modelling options for a component with flexibility.  This is primarily the blade and the support structure.
    
    Attributes
    ----------
    StructuralModellingType : StructuralModelling_StructuralModellingTypeEnum
        Defines the specific type of model in use.
    
    MaximumNodeSpacing : float
        The maximum node spacing allowed on the component.  If any two nodes are further spaced apart than this, an additional node or nodes will be added inbetween them.  If omite, no additional nodes will be added.
    
    GeometricStiffnessModel : StructuralModelling_GeometricStiffnessModelEnum, default='AxialLoadsOnly'
        The geometric stiffness model to use for the support structure
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - StructuralModellingInsert
        - ModalStructuralModelling
        - RigidStructuralModelling
    
    """
    StructuralModellingType: StructuralModelling_StructuralModellingTypeEnum = Field(alias="StructuralModellingType", default=None)
    MaximumNodeSpacing: float = Field(alias="MaximumNodeSpacing", default=None)
    GeometricStiffnessModel: StructuralModelling_GeometricStiffnessModelEnum = Field(alias="GeometricStiffnessModel", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


StructuralModelling.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.dynamic_wake import DynamicWake

from .schema_helper import SchemaHelper
from .models_impl import *


class DynamicWakeInsert(BladedEntity):
    r"""
    A DynamicWake is to be inserted from an external resource. The exact type of DynamicWake is not currently known.
    
    Attributes
    ----------
    DynamicWakeType : Literal['Insert'], default='Insert'
        For internal use when reading & writing JSON.
    
    insert : str
        A path to a resource from which a valid JSON model object can be resolved. All properties will be taken from the resolved object; no properties can be specified in-line.

    Notes
    -----
    
    """
    DynamicWakeType: Literal['Insert'] = Field(alias="DynamicWakeType", default='Insert', allow_mutation=False, const=True) # type: ignore
    insert: str = Field(alias="$insert", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'DynamicWakeType').merge(BladedEntity._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def is_insert(self) -> bool:
        """
        Returns true if the model is to be loaded from an external resource by the Bladed application; i.e. the 'insert' field is set with a resource location.
        """
        return True


    def _entity(self) -> bool:
        return True


DynamicWakeInsert.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.component import Component
from dnv_bladed_models.yaw_actuation import YawActuation
from dnv_bladed_models.yaw_bearing import YawBearing
from dnv_bladed_models.yaw_system_connectable_nodes import YawSystemConnectableNodes
from dnv_bladed_models.yaw_system_output_group_library import YawSystemOutputGroupLibrary

from .schema_helper import SchemaHelper
from .models_impl import *


class YawSystem(Component):
    r"""
    A definition of the yaw bearing and any activation.
    
    Attributes
    ----------
    ComponentType : Literal['YawSystem'], default='YawSystem'
        Defines the specific type of Component model in use.  For a `YawSystem` object, this must always be set to a value of `YawSystem`.
    
    Bearing : YawBearing
    
    Actuation : YawActuation
    
    OutputGroups : YawSystemOutputGroupLibrary, Not supported yet
    
    ConnectableNodes : YawSystemConnectableNodes, Not supported yet
    
    Notes
    -----
    
    """
    ComponentType: Literal['YawSystem'] = Field(alias="ComponentType", default='YawSystem', allow_mutation=False, const=True) # type: ignore
    Bearing: YawBearing = Field(alias="Bearing", default=None)
    Actuation: YawActuation = Field(alias="Actuation", default=None)
    OutputGroups: YawSystemOutputGroupLibrary = Field(alias="OutputGroups", default=YawSystemOutputGroupLibrary()) # Not supported yet
    ConnectableNodes: YawSystemConnectableNodes = Field(alias="ConnectableNodes", default=YawSystemConnectableNodes()) # Not supported yet

    _relative_schema_path = 'Components/YawSystem/YawSystem.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['OutputGroups','ConnectableNodes',]),
        'ComponentType').merge(Component._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


YawSystem.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.fixed_step_integrator import FixedStepIntegrator

from .schema_helper import SchemaHelper
from .models_impl import *


class ImplicitNewmarkBetaFixedStep(FixedStepIntegrator):
    r"""
    Settings for the Implicit Newmark Beta Fixed Step integrator.
    
    Attributes
    ----------
    IntegratorType : Literal['ImplicitNewmarkBetaFixedStep'], default='ImplicitNewmarkBetaFixedStep'
        Defines the specific type of Integrator model in use.  For a `ImplicitNewmarkBetaFixedStep` object, this must always be set to a value of `ImplicitNewmarkBetaFixedStep`.
    
    MaximumNumberOfIterations : int, default=1
        The maximum number of iterations for prescribed freedoms and first order states (e.g. dynamic stall & wake).  A value of 1 may sometimes inprecisely integrate first order states
    
    Beta : float, default=0.25
        The β parameter for the Newmark-β integration method.  The recommended value of 0.25 (with a γ value of 0.50) results in the constant average acceleration method that is unconditionally stable for linear systems.  A value of 0.26 (with a γ value of 0.52) results in a method that is close to the constant average acceleration method but includes a small amount of numerical damping to reduce unwanted vibrations of high-frequency modes. Note that the numerical damping increases with the step size.
    
    Gamma : float, default=0.5
        The γ parameter for the Newmark-β integration method.  The recommended value depends on the β parameter and given by the formula γ = 2.sqrt(β) - 0.5.  Values higher than 0.5 introduce positive numerical damping, whereas lower values introduce negative numerical damping.
    
    ToleranceMultiplier : float, default=1
        The tolerance used for defining the convergence criteria of the Newton-Raphson equilibrium iterations for 1st and 2nd order states.  Reduced values of this parameter result in more iterations and more accurate solutions, whereas increased values result in less iterations and less accurate solutions, which eventually may cause stability problems.  The allowable range of values for this parameter is 0.001 to 1000.
    
    Notes
    -----
    
    """
    IntegratorType: Literal['ImplicitNewmarkBetaFixedStep'] = Field(alias="IntegratorType", default='ImplicitNewmarkBetaFixedStep', allow_mutation=False, const=True) # type: ignore
    MaximumNumberOfIterations: int = Field(alias="MaximumNumberOfIterations", default=None)
    Beta: float = Field(alias="Beta", default=None)
    Gamma: float = Field(alias="Gamma", default=None)
    ToleranceMultiplier: float = Field(alias="ToleranceMultiplier", default=None)

    _relative_schema_path = 'Settings/SolverSettings/Integrator/ImplicitNewmarkBetaFixedStep.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'IntegratorType').merge(FixedStepIntegrator._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ImplicitNewmarkBetaFixedStep.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.perturbation_settings import PerturbationSettings

from .schema_helper import SchemaHelper
from .models_impl import *


class ModelLinearisationPerturbations(PerturbationSettings):
    r"""
    
    
    Not supported yet.
    
    Attributes
    ----------
    WindSpeedPerturbation : float, Not supported yet
        The magnitude of perturbation of the wind speed around the equilibrium value.  It should be small so that the analysis stays in the linear region  Omit keyword in order to let Bladed calculate a default perturbation.
    
    PitchPerturbation : float, Not supported yet
        The magnitude of perturbation of the pitch angle around the equilibrium value.  It should be small so that the analysis stays in the linear region.  Omit keyword in order to let Bladed calculate a default perturbation.
    
    GeneratorTorquePerturbation : float, Not supported yet
        The magnitude of perturbation of the generator torque around the equilibrium value.  It should be small so that the analysis stays in the linear region.  Omit keyword in order to let Bladed calculate a default perturbation.
    
    WindShearPerturbation : float, default=0, Not supported yet
        The magnitude of both horizontal and vertical shear perturbations   The default is to have no perturbation on the wind shear.
    
    ApplyPitchPerturbationToEachBlade : bool, default=False, Not supported yet
        If true, the pitch angle will be perturbed in turn for each blade as well as collectively for all blades
    
    YawActuatorTorquePerturbation : float, Not supported yet
        Yaw actuator torque perturbation. The torque perturbation is evenly distributed across the actuator banks.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    WindSpeedPerturbation: float = Field(alias="WindSpeedPerturbation", default=None) # Not supported yet
    PitchPerturbation: float = Field(alias="PitchPerturbation", default=None) # Not supported yet
    GeneratorTorquePerturbation: float = Field(alias="GeneratorTorquePerturbation", default=None) # Not supported yet
    WindShearPerturbation: float = Field(alias="WindShearPerturbation", default=None) # Not supported yet
    ApplyPitchPerturbationToEachBlade: bool = Field(alias="ApplyPitchPerturbationToEachBlade", default=None) # Not supported yet
    YawActuatorTorquePerturbation: float = Field(alias="YawActuatorTorquePerturbation", default=None) # Not supported yet

    _relative_schema_path = 'SteadyCalculation/ModelLinearisationPerturbations/ModelLinearisationPerturbations.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(PerturbationSettings._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ModelLinearisationPerturbations.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.yaw_fault import YawFault

from .schema_helper import SchemaHelper
from .models_impl import *


class YawFaultLimp(YawFault):
    r"""
    A fault in the yaw system where the yaw system is left without actuation.  The reaction torques are specified, and if these forces are overcome, the yaw bearing will move.
    
    Not supported yet.
    
    Attributes
    ----------
    EventType : Literal['YawFaultLimp'], default='YawFaultLimp', Not supported yet
        Defines the specific type of Event model in use.  For a `YawFaultLimp` object, this must always be set to a value of `YawFaultLimp`.
    
    Friction : float, Not supported yet
        The kinetic friction in the yaw bearing and any connected systems.
    
    Stiffness : float, Not supported yet
        The stiffness of the yaw system, if it has failed without actuation.
    
    Damping : float, Not supported yet
        The damping on the yaw system, if it has failed without actuation.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    EventType: Literal['YawFaultLimp'] = Field(alias="EventType", default='YawFaultLimp', allow_mutation=False, const=True) # Not supported yet # type: ignore
    Friction: float = Field(alias="Friction", default=None) # Not supported yet
    Stiffness: float = Field(alias="Stiffness", default=None) # Not supported yet
    Damping: float = Field(alias="Damping", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Event/YawFaultLimp.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'EventType').merge(YawFault._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


YawFaultLimp.update_forward_refs()
